package edu.harvard.mgh.purcell.gplink.data;

import java.io.IOException;

import org.apache.log4j.Logger;

import com.sshtools.j2ssh.ScpClient;
import com.sshtools.j2ssh.SshClient;

import edu.harvard.mgh.purcell.gplink.base_form.GeneralUtil;
import edu.harvard.mgh.purcell.gplink.mainGUI.ErrorDialog;
import edu.harvard.mgh.purcell.gplink.mainGUI.MainFrame;

/**
 * Upload a file(s) to a remote server via scp in an
 * independent thread.
 * @author Kathe Todd-Brown
 *
 */
public class UploadFile extends Thread {
	/**
	 * A string that is the remote directory.
	 */
	private String remoteFolder;
	/**
	 * An array of strings that hold the absolute local
	 * files that you want to upload.
	 */
	private String localFile[];
	/**
	 * An autherized connection that will be used in
	 * this upload.
	 */
	private SshClient conn;
	/**
	 * A MainFrame that this upload file is connected to.
	 */
	private MainFrame frame;
	
	/**
	 * Define a static logger variable so that it references the
	 */
	private static Logger logger = Logger.getLogger(UploadFile.class);

	/**
	 * Construct an upload thread.
	 * @param f the MainFrame this is attached to.
	 * @param getMe a String that is the local file to upload.
	 * @param putHere A String that is the remote folder to
	 * upload to.
	 * @param c A SshClient authorized connection.
	 */
	public UploadFile(MainFrame f, 
			String getMe, 
			String putHere, 
			SshClient c){
		super();
		frame = f;
		localFile = new String[1];
		localFile[0] = getMe;
		remoteFolder = putHere;
		conn = c;
	}
	/**
	 * Construct an upload thread.
	 * @param f the MainFrame this is attached to.
	 * @param getMe An array of Strings that is the local files to upload.
	 * @param putHere A String that is the remote folder to
	 * upload to.
	 * @param c A SshClient authorized connection.
	 */
	public UploadFile(MainFrame f, 
			String[] getMe, 
			String putHere, 
			SshClient c){
		super();
		frame = f;
		localFile = getMe;
		remoteFolder = putHere;
		conn = c;
	}
	
	/**
	 * Run the upload
	 */
	public void run(){
		//tell the logger we are starting
		logger.info("Start uploading.");
		//make the spaces in the remote folder hard
		remoteFolder = GeneralUtil.makeSpacesHard(remoteFolder);
		//initalize the scp client
		ScpClient scp = null;
		//try to open the scp connection
		try {
			scp = conn.openScpClient();
		//something messed up
		} catch (IOException e) {
			//tell the logger what happened
			logger.error("Cannot extablish scp client.");
			//re-establish the connection and notify the
			//...user that the connection is lost
			frame.data.connectionLost();
			//get out of here since the connection is lost
			return;
		}
		//notify the frame that there are files in transit
		frame.changeTransitFile(localFile, true);
		//put the local files in a string
		String localFiles = "";
		for(String file: localFile){
			localFiles = " ["+file+"]";
		}
		//notify the logger that we are uploading
		logger.info("Uploading "+
				localFiles+" to ["+remoteFolder+"].");
		//actually do the uploading
		try {
			scp.put(localFile, remoteFolder, false);
		} catch (IOException e) {
			//tell the user we couldn't upload the file(s)
			new ErrorDialog(frame, "Error trying upload "+
					localFiles+" to ["+remoteFolder+"].\n" +
			"run@UploadFile.java");
			//tell the logger we couldn't upload the files(s)
			logger.error("Error trying upload "+localFiles+
					" to ["+remoteFolder+"].");
		}
		//notify the frame that the files are done
		frame.changeTransitFile(localFile, false);
		//tell the logger we are leaving the uploader
		logger.info("End uploading");
	}

}
