package edu.harvard.mgh.purcell.gplink.mainGUI;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import edu.harvard.mgh.purcell.gplink.base_form.GeneralUtil.pickAFileButton;
import edu.harvard.mgh.purcell.gplink.data.Project;
import edu.harvard.mgh.purcell.gplink.data.UploadFile;

@SuppressWarnings("serial")
public class AddNonPlinkOpDialog extends JDialog {

	Project data;
	Vector <String> allOps;
	JTextField opName;
	JTextField opDisc;
	JTextField tempIn;
	JTextField tempOut;
	JTextField cline;
	JList infiles;
	DefaultListModel inlistmodel;
	JList outfiles;
	DefaultListModel outlistmodel;
	pickAFileButton addIn, addOut;
	JButton ok, cancel;
	JRadioButton runOp, badOp, goodOp;
	
	DocumentListener processIn = new DocumentListener(){
		public void changedUpdate(DocumentEvent arg0) {}

		public void insertUpdate(DocumentEvent arg0) {
			inlistmodel.addElement(tempIn.getText());
		}

		public void removeUpdate(DocumentEvent arg0) {}
	};
	DocumentListener processOut = new DocumentListener(){
		public void changedUpdate(DocumentEvent arg0) {}

		public void insertUpdate(DocumentEvent arg0) {
			outlistmodel.addElement(tempOut.getText());
		}

		public void removeUpdate(DocumentEvent arg0) {}
	};
	DocumentListener validateForm = new DocumentListener(){

		public void changedUpdate(DocumentEvent arg0) {
		}

		public void insertUpdate(DocumentEvent arg0) {
			ok.setEnabled(isvalid());
		}

		public void removeUpdate(DocumentEvent arg0) {
			ok.setEnabled(isvalid());
		}
		
	};
	
	ActionListener closeForm = new ActionListener(){
		public void actionPerformed(ActionEvent arg0) {
			AddNonPlinkOpDialog.this.dispose();
		}
	};
	
	ActionListener addOp = new ActionListener(){
		@SuppressWarnings("static-access")
		public void actionPerformed(ActionEvent e) {
			String [] infiles = new String [inlistmodel.getSize()];
			inlistmodel.copyInto(infiles);
			String [] outfiles = new String [outlistmodel.getSize()];
			outlistmodel.copyInto(outfiles);
			data.addOperation(cline.getText(), opName.getText(), 
					opDisc.getText(), infiles, new String [infiles.length], 
					outfiles, new String[outfiles.length]);
			data.frame.updateViews();
			
//			Assume that all imported operation are done and create
			//...a .gplink file if it's not already there.
			try {
				File gplinkfile = new  File(data.getRoot(), opName.getText()+ data.OP_STATUS_EXT);
				if(gplinkfile.createNewFile()){
					BufferedWriter out = new BufferedWriter(new FileWriter(gplinkfile));
				    out.write("0");
				    out.close();
				   
				    if( data.isRemote()){
				    	new UploadFile(data.frame, data.getRoot().getAbsolutePath() 
								+ File.separator + opName.getText() + data.OP_STATUS_EXT,
								data.getRemoteFile(), data.getConn()).start();
					}
				    
				}
			} catch (IOException e2) {
				new ErrorDialog(data.frame, "Error trying to create the .gplink file " +
						"for imported operation.\n process:ImportOpDialog()");
			}
			
			AddNonPlinkOpDialog.this.dispose();
		}
	};
	
	/**
	 * Check to see if the form is valid
	 * @return a boolean reflecting the validity
	 */
	private boolean isvalid(){
		//check to make sure the operation doesn't
		//...already exist
		String newOp = opName.getText();
		for(String opName: allOps){
			if(opName.equals(newOp))
				return false;
		}
		//make sure the name format is valid
		return newOp.matches("^[\\w\\-]+$");
	}
	
	private void buildLayout(){
		JPanel buttonPane = new JPanel();
		buttonPane.add(ok);
		buttonPane.add(cancel);
		
		JPanel root  = (JPanel)this.getContentPane();
		
		root.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		root.add(new JLabel("Operation name:"), c);
		c.gridx = 1;
		c.gridy = 0;
		c.weightx = 1;
		root.add(opName, c);
		c.gridx = 0;
		c.gridy ++;
		c.gridwidth = 1;
		root.add(new JLabel("Operation Discription (optional):"), c);
		c.gridx = 0;
		c.gridy ++;
		c.gridwidth = 2;
		root.add(opDisc, c);
		c.gridx = 0;
		c.gridy ++;
		c.gridwidth = 2;
		root.add(new JLabel("Command line (optional):"), c);
		c.gridx = 0;
		c.gridy ++;
		c.gridwidth = 2;
		root.add(cline, c);
		c.gridy ++;
		c.gridwidth = 1;
		root.add(new JLabel("Input files:"), c);
		c.gridy ++;
		c.gridwidth = 2;
		root.add(new JScrollPane(infiles), c);
		c.gridy ++;
		c.gridwidth = 1;
		c.gridx = 1;
		root.add(addIn, c);
		c.gridx = 0;
		c.gridy ++;
		c.gridwidth = 1;
		root.add(new JLabel("Output files:"), c);
		c.gridy ++;
		c.gridwidth = 2;
		root.add(new JScrollPane(outfiles), c);
		c.gridy ++;
		c.gridwidth = 1;
		c.gridx = 1;
		root.add(addOut, c);
		c.gridx = 0;
		c.gridy ++;
		c.gridwidth = 2;
		root.add(buttonPane, c);
	}
	
	public AddNonPlinkOpDialog(MainFrame mf){
		super(mf, "Import a Non-PLINK operation");
		
		
		data = mf.data;
		allOps = data.getAllOpName();
		tempIn = new JTextField("");
		tempIn.getDocument().addDocumentListener(processIn);
		tempOut = new JTextField("");
		tempOut.getDocument().addDocumentListener(processOut);
		opName = new JTextField(10);
		opName.getDocument().addDocumentListener(validateForm);
		opDisc = new JTextField(20);
		cline = new JTextField(20);
		outlistmodel = new DefaultListModel();
		inlistmodel = new DefaultListModel();
		infiles = new JList(inlistmodel);
		outfiles = new JList(outlistmodel);
		addIn = new pickAFileButton(tempIn, mf);
		addOut = new pickAFileButton(tempOut, mf);
		ok = new JButton("OK");
		ok.setEnabled(false);
		ok.addActionListener(addOp);
		cancel = new JButton("Cancel");
		cancel.addActionListener(closeForm);
		
		buildLayout();
	}
}
