package edu.harvard.mgh.purcell.gplink.mainGUI;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

//import ch.ethz.ssh2.SCPClient;
import edu.harvard.mgh.purcell.gplink.base_form.GeneralUtil;
import edu.harvard.mgh.purcell.gplink.data.DownloadFile;

/**
 * An abstract class that lays out the general popup 
 * menu that appears in both the operations
 * and folder viewers.
 * @author Kathe Todd-Brown
 *
 */
@SuppressWarnings("serial")
public abstract class GplinkPopUpMenu extends JPopupMenu {

	/**
	 * An array of the full file names of selected local files
	 */
	private String [] localFilenames;
	/**
	 * An array of the full file names of selected remote files
	 */
	private String [] remoteFilenames;
	/**
	 * An array of the operation names
	 */
	private String [] opnames;
	/**
	 * A mainframe this menu is attached to
	 */
	private MainFrame frame;
	/**
	 * Download selected files
	 */
	private ActionListener download = new ActionListener(){
		public void actionPerformed(ActionEvent e) {
			new DownloadFile(remoteFilenames, 
					frame.data.getRoot().getAbsolutePath(), 
					frame.data.getConn(),
					frame).start();
		}
	};
	/**
	 * Launch the user specified file viewer.
	 */
	private ActionListener openAlt = new ActionListener(){
		@SuppressWarnings("static-access")
		public void actionPerformed(ActionEvent event){
			//go through each localfile
			for(int i =0; i <localFilenames.length; i++){
				//check to see if it is still being downloaded
				if(frame.inTransit(localFilenames[i]) == false){
					try{
						//open the file in the editor
						String s = frame.data.getEditor() + " " 
								+ GeneralUtil.quoteFilename(localFilenames[i]);
						@SuppressWarnings("unused") 
						Process plink = Runtime.getRuntime().exec(GeneralUtil.stripAndSplit(s),
								null, frame.data.getRoot());
					}
			        catch (IOException e) {
			        	new ErrorDialog(frame, 
			        			"Error trying to open selected file [ "
			        			+ localFilenames[i] + " ] in " +
			        			"alternate editor [ " + frame.data.getEditor() + " ].\n");
			        }
		        } else {
		        	new ErrorDialog(frame, 
		        			"File still marked as downloading \n [ "
		        			+ localFilenames[i] + " ] \n"
		        			+ "Can not open.");
		        }
			}
		}
	};
	
	/**
	 * Launch the default file viewer
	 */
	private ActionListener openDefault = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			//go through each local file
			for(int i = 0; i < localFilenames.length; i++){
				//if the file isn't in transit
				if(frame.inTransit(localFilenames[i]) == false){
					try{
						//open the file in the user spcifed filename
						String s = frame.data.getDefaultEditor() + " "
							+ GeneralUtil.quoteFilename(localFilenames[i]);
						@SuppressWarnings("unused") 
						Process plink = Runtime.getRuntime().exec(GeneralUtil.stripAndSplit(s),
								null, frame.data.getRoot());
					}
			        catch (IOException e) {
			        	new ErrorDialog(frame, 
			        			"Error trying to open selected file [ "
			        			+ localFilenames[i] + " ] in " +
			        			"default editor [ " + frame.data.getDefaultEditor() + " ].\n");
			        }
		        
				} else {
					new ErrorDialog(frame, 
	        			"File still marked as downloading \n [ "
	        			+ localFilenames[i] + " ] \n"
	        			+ "Can not open.");
				}
			}
		}
	};
	
	/**
	 * Open local files in haploview
	 */
	private ActionListener openHaplo = new ActionListener(){
		public void actionPerformed(ActionEvent arg0) {
			//go through each local file selected
			for(int i = 0; i < localFilenames.length; i++){
				//make sure the file isn't in transit
				if(frame.inTransit(localFilenames[i]) == false){
					if(opnames == null)
						new OpenHapDialog(frame, null, localFilenames[i]);
					else
						//pass everything to a haploview dialog
						new OpenHapDialog(frame, opnames[i], localFilenames[i]);
				} else {
					new ErrorDialog(frame, 
	        			"File still marked as downloading \n [ "
	        			+ localFilenames[i] + " ] \n"
	        			+ "Can not open.");
				}
			}
		}
	};
	
	private ActionListener showHierachyOp = new ActionListener(){
		public void actionPerformed(ActionEvent arg0){
			new HierarchyDialog(opnames[0],
					frame, opnames[0], null);
		}
	};
	
	private ActionListener showHierachyFile = new ActionListener(){
		public void actionPerformed(ActionEvent arg0){
			if(frame.data.isRemote())
				new HierarchyDialog(GeneralUtil.fileName(remoteFilenames[0]),
					frame, null, remoteFilenames[0]);
			else
				new HierarchyDialog(GeneralUtil.fileName(localFilenames[0]),
						frame, null, localFilenames[0]);
		
		}
	};
	
	public GplinkPopUpMenu(MainFrame mf, 
			String [] givenFiles, String [] givenOp){
		super();
		opnames = givenOp;		
		frame = mf;
		
		boolean openFile = false;
		
		if(givenFiles != null && givenFiles.length != 0){
			if(frame.data.isflaggedRemote()){
				remoteFilenames = new String[givenFiles.length];
				for(int i = 0; i < givenFiles.length; i++){
					remoteFilenames[i] = frame.data.getRemoteFile() 
						+"/"+ GeneralUtil.fileName(givenFiles[i]);
				}
			}
			localFilenames = new String[givenFiles.length];
			for(int i = 0; i < givenFiles.length; i++){
				localFilenames[i] = new File(frame.data.getRoot(),
						GeneralUtil.fileName(givenFiles[i])).getAbsolutePath();
			}
			openFile = true;
		
			if(openFile == true){
				JMenuItem getFile = new JMenuItem("Download file");
				getFile.addActionListener(download);
		
				//check that the file isn't already there
				boolean localCopy = true;
				for(int i = 0; i < localFilenames.length; i++){
					if( ! new File(localFilenames[i]).exists())
						localCopy = false;
				}
				
				if(frame.data.isRemote()
						&& localCopy == false){
					//add download to the menu
					add(getFile);
					add(new JPopupMenu.Separator());
				}
				
				if(localCopy == true){
					JMenuItem open = new JMenuItem("Open in default viewer");
					open.addActionListener(openDefault);
					add(open);
					
					JMenuItem openAlter = new JMenuItem("Open in alternate viewer");
					openAlter.addActionListener(openAlt);
					add(openAlter);
					
					boolean hapOK = true;
					//if not all the files are haploview ok
					for(int i = 0; i < localFilenames.length; i++){
						if( !OpenHapDialog.hapoloviewOK(localFilenames[i]))
							hapOK = false;
					}
					
					if(hapOK == true
							&& new File(frame.data.getHaploviewPath()).exists() ){
						JMenuItem openInHaplo = new JMenuItem("Open in Haploview");
						openInHaplo.addActionListener(openHaplo);
						add(openInHaplo); 
					}
					add(new JPopupMenu.Separator());
				}
				
			}
			
			JMenuItem fileParents;
			if(frame.data.isflaggedRemote())
				fileParents = new JMenuItem("Show relations of "+GeneralUtil.fileName(remoteFilenames[0]));
			else
				fileParents = new JMenuItem("Show relations of "+GeneralUtil.fileName(localFilenames[0]));
			fileParents.addActionListener(showHierachyFile);
			
			add(fileParents);
				
		}
		
		
		if(opnames != null){
			JMenuItem opParents = new JMenuItem("Show relations of " + opnames[0]);
			opParents.addActionListener(showHierachyOp);
			add(opParents);
		}
		add(new JPopupMenu.Separator());
	}
	
}
