package edu.harvard.mgh.purcell.gplink.mainGUI;

import java.io.*;
import java.util.HashSet;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JButton;

//import ch.ethz.ssh2.Session;

import edu.harvard.mgh.purcell.gplink.data.Project;
import edu.harvard.mgh.purcell.gplink.data.SendCommand;
import edu.harvard.mgh.purcell.gplink.base_form.GeneralUtil;
/**
 * PlinkExicute is a simple JDialog which processes a plink command passed
 * to it.
 * @author ktoddbrown
 */
@SuppressWarnings("serial")
public class PlinkExicute extends JDialog {
	
	
	//labels that tell the user what's what
	static private JLabel COMMAND_LABEL = new JLabel ("PLINK command: ");
	static private JLabel DISCR_LABEL = new JLabel ("Command description: ");
	
	//editable text fields that hold the name, command and discription
	//...of the plink command
	private JTextField command = new JTextField(50);
	private JTextField discription = new JTextField(50);	
	
	//the project which is used to store the operation if it is run
	private Project data;
	
	//the main application folder which we use to update the
	//...operations viewer
	private MainFrame frame;
	
	
	/**
	 * runs the plink command and stores it in the project data.
	 */
	ActionListener runPLINK = new ActionListener(){
		public void actionPerformed(ActionEvent event){
			
			String cmd = command.getText();
			if(!cmd.endsWith(" --gplink")){
				cmd = cmd + " --gplink";
			}
			
			//a string of the names of the input files
			String[] inputnames = getInput(cmd);
			
//			check to see if there is a log file already with
			//...this output base. If there is then we quit!
			String logName = getLogfile(cmd);
			if(new File(logName).exists()){
				new PlinkExicute(frame, cmd);
				JOptionPane.showMessageDialog(frame, 
						logName + " already exists. \n" +
						"Please pick a new name using --out.", 
						"gPLINK error",
	        			JOptionPane.ERROR_MESSAGE);
				dispose();
				return;
			}
			
			if(logName.equals("")){
				new PlinkExicute(frame, cmd);
				JOptionPane.showMessageDialog(frame, 
						"Output name not found in command. " +
						"gPLINK commands must use" +
						" the --out flag.",
						"gPLINK Error",
	        			JOptionPane.ERROR_MESSAGE);
				dispose();
				return;
			}
			
			//a string of the names of the output files
			String[] outputnames = {logName};
				
				if(data.isRemote()){
					cmd =  GeneralUtil.makeQuotedSpacesHard(cmd);
					new SendCommand(cmd, data).execute();
					
				}else{
					try{
						@SuppressWarnings("unused")
						Process plink = Runtime.getRuntime().exec(
								GeneralUtil.stripAndSplit(cmd) 
								, null, data.getRoot());
				
					}
			        catch (IOException e) {
			        	new ErrorDialog(frame, 
			        			"Error executing PLINK command. Please " +
			        			"go back and check that the command " +
			        			"is a valid PLINK command.\n" +
			        			cmd);
			        	return;
			        }
			}
			
	        // update the log viewer
            frame.logViewer.update(logName, false);
            	        
	        //set all the input discriptions blank
	        String[] inputdisc = new String[inputnames.length];
	        for(int i = 0; i < inputnames.length; i++)
	        	inputdisc[i] = "";
	        
	        //set all the output discriptions blank
	        String[] outputdisc = new String[outputnames.length];
	        for(int i = 0; i < outputnames.length; i++)
	        	outputdisc[i] = "";
	        	
	        //add the operation to the project
	        data.addOperation(cmd,
	        		new File(getOutputRoot(cmd)).getName(),
	        		discription.getText(),
					inputnames, inputdisc, 
					outputnames, outputdisc);
			
	        //update the operations view in the main application frame
			frame.updateViews();
			
			//close the window
			dispose();
		}
	};
	
	String getOutputRoot(String cline){
		String ans = "";

		//dump the command line to an array splitting on spacing
		String [] commandArray = GeneralUtil.stripAndSplit(cline);
		for(int i = 0; i < commandArray.length; i ++){
			//check to see if the index is at the output flag
			if((commandArray[i]).equals("--out") 
					&& (i+1) < commandArray.length)
				ans = commandArray[i+1];
								
		}

		if(ans.equals(""))
			ans = "plink";
			
		if(new File(ans).getParent() == null){
			if(data.isflaggedRemote())
				ans = data.getRemoteFile() + "/"
				+ ans;
			else
				ans = data.getRoot().getAbsolutePath()
				+ File.separator + ans;
		}
		
		return ans;
	}
	
	/**
	 * Pulls the name of the log file from a command line
	 * @param cline a command line
	 * @return a string which identifies the log file
	 */
	String getLogfile(String cline){
		return getOutputRoot(cline) + ".log";
	}
	
	/**
	 * pulls an array of strings identifing the input files
	 * @param cline a command line string
	 * @return an array of strings identifying the input files
	 */
	String [] getInput(String cline){
		
		//a hash set that allows us to dynamically dump the input files
		//...we find, it also has the advantage of not double listing
		//...the input files
		HashSet<String> dynamicAns = new HashSet<String>();
		//split the commandline up based on empty spaces outside of quotes
		//...and strip the quotes
		String [] commandArray = GeneralUtil.stripAndSplit(cline);
		
		//go through the command
		for(int i = 0; i < commandArray.length; i ++){
			if(commandArray[i].contains(".") 
					&& (!commandArray[i].matches("\\d*.?\\d*"))
					&& (!commandArray[i].endsWith(".exe"))
					&& (!commandArray[i].endsWith(".exe\""))
					&& (!commandArray[i].endsWith("plink"))
					&& (!commandArray[i].endsWith("plink\""))){
				if(data.isflaggedRemote())
					dynamicAns.add(commandArray[i]);
				else if(new File(data.getRoot() + File.separator 
							+ commandArray[i]).exists())
					dynamicAns.add(data.getRoot() + File.separator 
							+ commandArray[i]);
				else
					dynamicAns.add(commandArray[i]);
			}
			
			if(commandArray[i].equals("--bfile")){
				String tempRoot = "plink";
				if( (i+1) < commandArray.length)
					tempRoot = commandArray[i+1];
			
				if(data.isflaggedRemote() == true)
					dynamicAns.add(tempRoot + ".bed");
				else
					dynamicAns.add(data.getRoot() + File.separator 
							+tempRoot + ".bed");
				
				if(data.isflaggedRemote() == true)
					dynamicAns.add(tempRoot + ".fam");
				else
					dynamicAns.add(data.getRoot() + File.separator 
						+tempRoot + ".fam");
				
				if(data.isflaggedRemote() == true)
					dynamicAns.add(tempRoot + ".bim");
				else
					dynamicAns.add(data.getRoot() + File.separator 
							+tempRoot + ".bim");
			}
			
			if(commandArray[i].equals("--file")){
				String tempRoot = "plink";
				if( (i+1) < commandArray.length)
					tempRoot = commandArray[i+1];
				
				if(data.isflaggedRemote() == true)
					dynamicAns.add(tempRoot + ".map");
				else
					dynamicAns.add(data.getRoot() + File.separator 
							+tempRoot + ".map");

				if(data.isflaggedRemote() == true)
					dynamicAns.add(tempRoot + ".ped");
				else
					dynamicAns.add(data.getRoot() + File.separator 
							+ tempRoot + ".ped");
			}
		}
			
		//make the answer
		int numInfiles = dynamicAns.size();
		String [] ans = new String [numInfiles];
		
		//copy the answer from the hash set
		int index = 0;
		for(String s: dynamicAns){
			ans[index] = s;
			index ++;
		}
		
		//return the answer
		return ans;
	}
	
	/**
	 * PlinkExicute creates a dialog where the user can edit the name
	 * command and discription of a plink command
	 * @param mf the main frame this application is running form
	 * @param cline the command line we are going to exicute
	 */
	public PlinkExicute(MainFrame mf, String cline){
		//create a new JDialog tied to the main frame and give it a title
		super(mf, "Execute Command");
		//layout this new dialog
		setLayout(new GridBagLayout());
		GridBagConstraints layoutConstraints = new GridBagConstraints();
		layoutConstraints.fill = GridBagConstraints.HORIZONTAL;
		layoutConstraints.weightx = 1;
		layoutConstraints.weighty = 1;
		//create a button to exicute this command
		JButton exicute = new JButton("Run");
		//we actually don't do any error checking for the legitimacy of 
		//...this command other then the in/output files
		exicute.addActionListener(runPLINK);
		
		//set all the parts
		frame = mf;
		//inputnames = i;
		//outputnames = o;
		data = frame.data;
		//command = new JTextField(cline, 80);
		command.setText(cline);
		//generate a new name for the operation based on the total number
		//...of operations in the project
		//name.setText("temp_op_"+ data.getNumOp());
		//File temp = new File(getOutputRoot(cline));
		//name.setText(temp.getName());
		
		layoutConstraints.gridx = 0;
		layoutConstraints.gridy = 0;
		//NAME_LABEL.setAlignmentX(Component.RIGHT_ALIGNMENT);
		//add everything to the dialog
		//add(NAME_LABEL, layoutConstraints);
		layoutConstraints.gridx = 1;
		layoutConstraints.gridy = 0;
		//add(name, layoutConstraints);
		layoutConstraints.gridx = 0;
		layoutConstraints.gridy = 0;
		COMMAND_LABEL.setAlignmentX(Component.RIGHT_ALIGNMENT);
		add(COMMAND_LABEL, layoutConstraints);
		layoutConstraints.gridx = 1;
		layoutConstraints.gridy = 0;
		add(command, layoutConstraints);
		layoutConstraints.gridx = 0;
		layoutConstraints.gridy = 1;
		DISCR_LABEL.setAlignmentX(Component.RIGHT_ALIGNMENT);
		add(DISCR_LABEL, layoutConstraints);
		layoutConstraints.gridx = 1;
		layoutConstraints.gridy = 1;
		add(discription, layoutConstraints);
		layoutConstraints.gridx = 0;
		layoutConstraints.gridy = 2;
		add(exicute, layoutConstraints);
		
		//pack it
		pack();
		//send it off!
		setVisible(true);
		
	}
	

}
