package edu.harvard.mgh.purcell.gplink.mainGUI;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.IOException;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

/**
 * An extention of JDialog that get and sets the 
 * project folder information.
 * @author Kathe Todd-Brown
 *
 */
@SuppressWarnings("serial")
public class ProjectDialog extends JDialog {
	/**
	 * The main window.
	 */
	private MainFrame frame;
	/**
	 * A JTextField that holds the local folder name.
	 */
	private JTextField localFolder;
	/**
	 * A JTextField that holds the user name.
	 */
	private JTextField user;
	/**
	 * A JTextField that holds the host name.
	 */
	private JTextField host;
	/**
	 * A JTextField that holds the port number.
	 */
	private JTextField port;
	/**
	 * A JTextField that holds the remote folder name.
	 */
	private JTextField remoteFolder;
	/**
	 * The browse button attached to the remote folder name.
	 */
	JButton browseRemote;
	/**
	 * The browse button attached to the local folder name.
	 */
	JButton browseLocal;
	/**
	 * The check box that indicates a remote project.
	 */
	JCheckBox flagRemote;
	/**
	 * Section of the form that is pertinate to the remote
	 * parameters.
	 */
	JPanel remotePanel;
	/**
	 * Button for logging into server.
	 */
	JButton login;
	/**
	 * Button for completion of the form.
	 */
	JButton done;
	/**
	 * Button to cancel the form.
	 */
	JButton cancel;
	
	/**
	 * A boolean function that checks to see if the
	 * form is completed.
	 * @return true if the form is completed, false otherwise
	 */
	private boolean check(){
		//check that there is something in the local folder
		if(localFolder.getText() == ""){
			return false;
		}
		//if the remote folder is flagged then check
		//...that none of the fields are empty
		if(flagRemote.isSelected()
				&& (remoteFolder.getText().equals("")
						|| user.getText().equals("")
						|| host.getText().equals("")
						|| port.getText().equals("")
						|| frame.data.getConn() == null
						|| !frame.data.getConn().isAuthenticated())
						){
			return false;
		}
		
		//we haven't tripped anything!
		return true;
	}
	
	/**
	 * Check to see if we have completed the form when
	 * each text field is modified.
	 */
	DocumentListener checkValid = new DocumentListener(){
		public void changedUpdate(DocumentEvent arg0) {
			login.setEnabled(!(host.getText().equals("")
					|| user.getText().equals("")
					|| port.getText().equals("")));
			browseRemote.setEnabled(frame.data.getConn() != null);
			done.setEnabled(check());
		}

		public void insertUpdate(DocumentEvent arg0) {
			login.setEnabled(!(host.getText().equals("")
					|| user.getText().equals("")
					|| port.getText().equals("")));
			browseRemote.setEnabled(frame.data.getConn() != null);
			done.setEnabled(check());
		}

		public void removeUpdate(DocumentEvent arg0) {
			login.setEnabled(!(host.getText().equals("")
					|| user.getText().equals("")
					|| port.getText().equals("")));
			browseRemote.setEnabled(frame.data.getConn() != null);
			done.setEnabled(check());
		}
		
	};
	
	
	
	/**
	 * launch a JFileChooser to set the local directory
	 */
	ActionListener getLocal = new ActionListener(){
		public void actionPerformed(ActionEvent arg0) {
			//create a new dialog to pick the directory
			JFileChooser fc = new JFileChooser();
			// only look at directories
			fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			// show the dialog and when it is OK-ed
			if (fc.showOpenDialog(ProjectDialog.this) 
					== JFileChooser.APPROVE_OPTION) {
				localFolder.setText(fc.getSelectedFile()
						.getAbsolutePath());
			}
			//make sure the user can now opt for a remote project
			flagRemote.setEnabled(true);
		}
	};
	
	/**
	 * launch a RemoteFileChooser to set a remote 
	 * directory
	 */
	ActionListener getRemote = new ActionListener(){

		public void actionPerformed(ActionEvent arg0) {
			new RemoteFileChooser(ProjectDialog.this, 
					frame.data, 
					remoteFolder, "", true);			
		}
		
	};
	/**
	 * close everything up
	 */
	private void closing(){
		frame.data.closeOldProject();
		frame.updateViews();
		dispose();
	}
	
	/**
	 * close everything up if cancel is clicked
	 */
	ActionListener cancelAction = new ActionListener(){
		public void actionPerformed(ActionEvent e) {
			closing();}
	};
	/**
	 * close everything up if x is clicked
	 */
	WindowListener properClosing = new WindowListener(){
		public void windowActivated(WindowEvent e) {}
		public void windowClosed(WindowEvent e) {
			closing();}
		public void windowClosing(WindowEvent e) {
			closing();}
		public void windowDeactivated(WindowEvent e) {}
		public void windowDeiconified(WindowEvent e) {}
		public void windowIconified(WindowEvent e) {}
		public void windowOpened(WindowEvent e) {}
		
	};
	
	/**
	 * Set the new project up.
	 */
	ActionListener setProject = new ActionListener(){
		@SuppressWarnings("static-access")
		public void actionPerformed(ActionEvent e) {
			frame.setStatus("Setting the project folder...");
			
			//check that there is an existing folder
			if(! new File(localFolder.getText()).isDirectory()){
				new ErrorDialog(frame, "You have selected a local project " +
						"folder that does not exist.\n Please create the folder and try again.");
				return;
			}
				
			//if the project is flaged as remote
			if(flagRemote.isSelected() == true){
				//System.out.println("Flag2 setProject.actionPreformed()@ProjectDialog.java "
				//		+ (new GregorianCalendar()).getTimeInMillis());
				
				
				//set the remote file
				frame.data.setRemoteFile(new File(localFolder.getText()),
						remoteFolder.getText());
				
				//System.out.println("Flag3 setProject.actionPreformed()@ProjectDialog.java "
				//		+ (new GregorianCalendar()).getTimeInMillis());
			}else
				
				//set the root file
				frame.data.setLocalProject(new File(localFolder.getText()));
			
			//if there isn't a configuration file then pop-up the dialog
			if(! new File(frame.data.getRoot(), frame.data.LOG_FILE).exists())
				new ConfigDialog(frame);
			
			// set some menu to enabled since we now have a root folder
			frame.menuStuff.setMenuLock(frame.data.hasMetaLock());
			frame.menuStuff.options.setEnabled(true);
			frame.menuStuff.export.setEnabled(true);
			//System.out.println("Flag4 setProject.actionPreformed()@ProjectDialog.java "
			//		+ (new GregorianCalendar()).getTimeInMillis());
		
			// update the operations panel
			frame.updateViews();
			
			frame.setStatus("Ready!");
			dispose();
		}
	};
	
	ItemListener openRemotePane = new ItemListener(){
		@SuppressWarnings("static-access")
		public void itemStateChanged(ItemEvent e) {
			if(flagRemote.isSelected())
				try {
					new File(localFolder.getText(), frame.data.REMOTE_FILE).createNewFile();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			else
				new File(localFolder.getText(), frame.data.REMOTE_FILE).delete();
			
			done.setEnabled(check());
			ProjectDialog.this.remotePanel.setVisible(flagRemote.isSelected());
			pack();
		}
	};
	
	ActionListener launchLogin = new ActionListener(){
		@SuppressWarnings("static-access")
		public void actionPerformed(ActionEvent e1) {
			//set everything up
			frame.data.setUser(user.getText());
			frame.data.setHost(host.getText());
			frame.data.setPort(new Integer(port.getText()));
			//connect and enable the remote folder chooser if
			//...we establish a connection
			browseRemote.setEnabled(frame.data.connect());
			done.setEnabled(check());
		}
	};
	
	public ProjectDialog(MainFrame mf, String localname,
			String remotename, String hostname, 
			String username, int portnum){
		//this(mf, true);
		this(mf, false);
		//this.setVisible(false);
		localFolder.setText(localname);
		flagRemote.setEnabled(true);
		remoteFolder.setText(remotename);
		user.setText(username);
		host.setText(hostname);
		port.setText((new Integer(portnum)).toString());
		
		flagRemote.setSelected(true);
		
		pack();
		setVisible(true);
	}
	
	public ProjectDialog(MainFrame mf){
		this(mf, true);
	}
	
	public ProjectDialog(MainFrame mf, String local){
		this(mf, false);
		localFolder.setText(local);
		flagRemote.setEnabled(true);
		pack();
		setVisible(true);
	}
	
	public ProjectDialog(MainFrame mf, boolean visible){
		super(mf, "Set the project folder", true);
		frame = mf;
		
		frame.data.closeOldProject();
		
		init();
		
		//set up the local panel
		JPanel localPanel = new JPanel();
		localPanel.add(new JLabel("Local Folder:"));
		localPanel.add(localFolder);
		localPanel.add(browseLocal);
		
		//set up the remote panel
		remotePanel = new JPanel();
		remotePanel.setLayout(new BoxLayout( 
				remotePanel, BoxLayout.PAGE_AXIS));
		JPanel loginPanel = new JPanel();
		loginPanel.add(new JLabel("User:"));
		loginPanel.add(user);
		loginPanel.add(new JLabel("Host:"));
		loginPanel.add(host);
		loginPanel.add(new JLabel("Port:"));
		loginPanel.add(port);
		loginPanel.add(login);
		JPanel remoteFolderPanel = new JPanel();
		remoteFolderPanel.add(new JLabel("Remote Folder:"));
		remoteFolderPanel.add(remoteFolder);
		remoteFolderPanel.add(browseRemote);
		remotePanel.add(loginPanel);
		remotePanel.add(remoteFolderPanel);
		remotePanel.setVisible(false);
		
		//set up the buttons
		JPanel buttonPanel = new JPanel();
		done.setAlignmentY(Component.LEFT_ALIGNMENT);
		buttonPanel.add(done);
		cancel.setAlignmentY(Component.LEFT_ALIGNMENT);
		buttonPanel.add(cancel);
		
		//throw it all together
		this.getContentPane().setLayout(new BoxLayout( 
				this.getContentPane(), BoxLayout.PAGE_AXIS));
		this.getContentPane().add(localPanel);
		this.getContentPane().add(flagRemote);
		this.getContentPane().add(new JSeparator());
		this.getContentPane().add(remotePanel);
		this.getContentPane().add(buttonPanel);
		
		pack();
		setVisible(visible);
	}
	
	private void init(){

		user = new JTextField(10);
		user.getDocument().addDocumentListener(checkValid);
		host = new JTextField(30);
		host.getDocument().addDocumentListener(checkValid);
		port = new JTextField(5);
		port.setText("22");
		port.getDocument().addDocumentListener(checkValid);
		localFolder = new JTextField(30);
		localFolder.getDocument().addDocumentListener(checkValid);
		localFolder.setEditable(false);
		remoteFolder = new JTextField(30);
		remoteFolder.getDocument().addDocumentListener(checkValid);
		remoteFolder.setEditable(false);
		browseLocal = new JButton("Browse");
		browseRemote = new JButton("Browse");
		browseLocal.addActionListener(getLocal);
		browseRemote.addActionListener(getRemote);
		
		cancel = new JButton("Cancel");
		cancel.addActionListener(cancelAction);
		
		done = new JButton("Done");
		done.addActionListener(setProject);
		done.setEnabled(false);
		
		login = new JButton("Login");
		login.addActionListener(launchLogin);
		login.setEnabled(false);
		
		flagRemote = new JCheckBox("Link to a remote project " +
				"(via ssh to a Linux/Unix host)\n " +
				"Note this is in developement and currently unsupported");
		flagRemote.addItemListener(openRemotePane);
	}

}
